<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       http://preventdirectaccess.com/
 * @since      1.0.0
 *
 * @package    Pda_Contact_Forms
 * @subpackage Pda_Contact_Forms/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Pda_Contact_Forms
 * @subpackage Pda_Contact_Forms/includes
 * @author     BWPS <hello@preventdirectaccess.com>
 */
class Pda_Contact_Forms {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Pda_Contact_Forms_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'PDA_CONTACT_FORMS_VERSION' ) ) {
			$this->version = PDA_CONTACT_FORMS_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'PDA Contact Forms';
        $this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Pda_Contact_Forms_Loader. Orchestrates the hooks of the plugin.
	 * - Pda_Contact_Forms_i18n. Defines internationalization functionality.
	 * - Pda_Contact_Forms_Admin. Defines all hooks for the admin area.
	 * - Pda_Contact_Forms_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pda-contact-forms-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pda-contact-forms-i18n.php';

		/**
		 * The class responsible for defining the constants
		 */
		require_once plugin_dir_path( dirname( __FILE__) ) . 'includes/class-pda-contact-forms-constant.php';

        /**
         * The class responsible for utilities functions
         */
		require_once plugin_dir_path( dirname( __FILE__) ) . 'includes/class-pda-contact-forms-utils.php';
		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-pda-contact-forms-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-pda-contact-forms-public.php';

		/**
         * The class responsible for Ymese helper classes
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'yme-wp-plugins-sdk/require.php';

		$this->loader = new Pda_Contact_Forms_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Pda_Contact_Forms_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Pda_Contact_Forms_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Pda_Contact_Forms_Admin( $this->get_plugin_name(), $this->get_version() );

        $this->loader->add_action( 'admin_notices', $plugin_admin, 'check_plugin_pda_gold_activate' );
        $this->loader->add_action( 'pda_contact_forms', $plugin_admin, 'render_contact_forms' );

        $this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
        $this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
        $this->loader->add_action( 'ccf_successful_submission', $plugin_admin, 'get_post_id_file_in_forms', 10, 2);
        $this->loader->add_action( 'wpcf7_before_send_mail', $plugin_admin, 'protect_file_by_contact_form_7', 10, 1);
        $this->loader->add_action('wp_ajax_contact_forms_update_settings', $plugin_admin, 'contact_forms_update_settings');

        $this->loader->add_action('ninja_forms_after_submission', $plugin_admin, 'protect_file_by_ninja_forms_files_upload');

        $this->loader->add_action('save_post', $plugin_admin, 'pda_contact_forms_handle_save_post', 10, 1);

        $this->loader->add_filter( 'add_attachment', $plugin_admin, 'pcf_handle_upload', 10, 1 );

        //ACF
        $this->loader->add_action('acf/render_field_settings/type=repeater', $plugin_admin,'render_field_settings');
        $this->loader->add_action('ajax_query_attachments_args', $plugin_admin,'pcf_filter_unprotected_files');

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Pda_Contact_Forms_Public( $this->get_plugin_name(), $this->get_version() );

        $this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
        $this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Pda_Contact_Forms_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
