<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       www.buildwps.com/prevent-direct-access-premium-plugin/?utm_source=user-website&utm_medium=pluginpage&utm_campaign=plugin-author-link
 * @since      1.0.0
 *
 * @package    Pda_Wp_Magic_Link
 * @subpackage Pda_Wp_Magic_Link/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Pda_Wp_Magic_Link
 * @subpackage Pda_Wp_Magic_Link/includes
 * @author     buildwps <hello@ymese.com>
 */
class Pda_Wp_Magic_Link {

    /**
     * The loader that's responsible for maintaining and registering all hooks that power
     * the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      Pda_Wp_Magic_Link_Loader $loader Maintains and registers all hooks for the plugin.
     */
    protected $loader;

    /**
     * The unique identifier of this plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string $plugin_name The string used to uniquely identify this plugin.
     */
    protected $plugin_name;

    /**
     * The current version of the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string $version The current version of the plugin.
     */
    protected $version;

    /**
     * Define the core functionality of the plugin.
     *
     * Set the plugin name and the plugin version that can be used throughout the plugin.
     * Load the dependencies, define the locale, and set the hooks for the admin area and
     * the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function __construct() {
        if ( defined( 'PML_VERSION' ) ) {
            $this->version = PML_VERSION;
        } else {
            $this->version = '1.0.0';
        }
//		$this->plugin_name = 'pda-wp-magic-link';
        $this->plugin_name = 'PDA Private Magic Links';

        $this->load_dependencies();
        $this->set_locale();
        $this->define_admin_hooks();
        $this->define_public_hooks();

    }

    /**
     * Load the required dependencies for this plugin.
     *
     * Include the following files that make up the plugin:
     *
     * - Pda_Wp_Magic_Link_Loader. Orchestrates the hooks of the plugin.
     * - Pda_Wp_Magic_Link_i18n. Defines internationalization functionality.
     * - Pda_Wp_Magic_Link_Admin. Defines all hooks for the admin area.
     * - Pda_Wp_Magic_Link_Public. Defines all hooks for the public side of the site.
     *
     * Create an instance of the loader which will be used to register the hooks
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function load_dependencies() {

        /**
         * The class responsible for utils function
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pda-wp-magic-link-utils.php';

        /**
         * The class responsible for orchestrating the actions and filters of the
         * core plugin.
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pda-wp-magic-link-loader.php';

        /**
         * The class responsible for defining internationalization functionality
         * of the plugin.
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pda-wp-magic-link-i18n.php';

        /**
         * Constants
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pda-wp-magic-link-constants.php';

        /**
         * DB
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pda-wp-magic-link-db.php';

        /**
         * The class responsible for defining all actions that occur in the admin area.
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-pda-wp-magic-link-admin.php';

        /**
         * The class responsible for defining all actions that occur in the public-facing
         * side of the site.
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-pda-wp-magic-link-public.php';

        /**
         * This class responsible for require all sdk helper classes
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'yme-wp-plugins-sdk/require.php';

        /**
         * This class responsible for database interaction
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pda-wp-magic-link-repo.php';

        /**
         *
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pda-wp-magic-link-functions.php';

        $this->loader = new Pda_Wp_Magic_Link_Loader();

    }

    /**
     * Define the locale for this plugin for internationalization.
     *
     * Uses the Pda_Wp_Magic_Link_i18n class in order to set the domain and to register the hook
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function set_locale() {

        $plugin_i18n = new Pda_Wp_Magic_Link_i18n();

        $this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

    }

    /**
     * Register all of the hooks related to the admin area functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_admin_hooks() {

        $plugin_admin = new Pda_Wp_Magic_Link_Admin( $this->get_plugin_name(), $this->get_version() );

        $this->loader->add_action( 'admin_init', $plugin_admin, 'handle_admin_init' );
        $this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
        $this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
        $this->loader->add_action( 'admin_notices', $plugin_admin, 'pda_wml_admin_notice' );
        $this->loader->add_action( 'init', $plugin_admin, 'add_magiclink_button' );
        $this->loader->add_shortcode( 'magic_link', $plugin_admin, 'magiclink_shortcode' );

        $this->loader->add_filter( 'the_content', $plugin_admin, 'filter_protected_file', 9999999 );
	    $this->loader->add_filter( 'wp_calculate_image_srcset', $plugin_admin, 'handle_calculate_image_srcset', 10, 5 );
	    $this->loader->add_filter( 'wp_calculate_image_srcset_meta', $plugin_admin, 'handle_calculate_image_srcset_meta', 10, 4 );

        $this->loader->add_filter( 'media_send_to_editor', $plugin_admin, 'custom_widget_text_content' );

        $this->loader->add_action( 'pda_magic_link_settings', $plugin_admin, 'pda_render_magic_link_settings' );

        $this->loader->add_action( 'ppp_magic_link_settings', $plugin_admin, 'ppp_render_magic_link_settings' );

        $this->loader->add_action( 'wp_ajax_update_option_setting_magic', $plugin_admin, 'update_option_setting_magic' );

        $this->loader->add_filter( "pda_magic_link_get_default_private_link", $plugin_admin, 'query_get_default_private_link', 10, 3 );

//        $this->loader->add_filter( "pda_magic_link_insert_default_private_link", $plugin_admin, 'insert_default_private_link_for_pda', 10, 3 );

        $this->loader->add_filter( "pdav3_check_user_access_private_link", $plugin_admin, 'check_user_access_private_link', 10, 1 );

        $this->loader->add_filter( 'pda_v3_handle_file_return_for_private_link', $plugin_admin, 'handle_s3_type_for_user_private_link', 10, 2 );

	    $this->loader->add_filter( 'post_thumbnail_html', $plugin_admin, 'handle_before_render_feature_image', 10 );
    }

    /**
     * Register all of the hooks related to the public-facing functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_public_hooks() {

        $plugin_public = new Pda_Wp_Magic_Link_Public( $this->get_plugin_name(), $this->get_version() );

        $this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
        $this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

    }

    /**
     * Run the loader to execute all of the hooks with WordPress.
     *
     * @since    1.0.0
     */
    public function run() {
        $this->loader->run();
    }

    /**
     * The name of the plugin used to uniquely identify it within the context of
     * WordPress and to define internationalization functionality.
     *
     * @since     1.0.0
     * @return    string    The name of the plugin.
     */
    public function get_plugin_name() {
        return $this->plugin_name;
    }

    /**
     * The reference to the class that orchestrates the hooks with the plugin.
     *
     * @since     1.0.0
     * @return    Pda_Wp_Magic_Link_Loader    Orchestrates the hooks of the plugin.
     */
    public function get_loader() {
        return $this->loader;
    }

    /**
     * Retrieve the version number of the plugin.
     *
     * @since     1.0.0
     * @return    string    The version number of the plugin.
     */
    public function get_version() {
        return $this->version;
    }

}
