<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       preventdirectaccess.com
 * @since      1.0.0
 *
 * @package    Pda_Membership_Integration
 * @subpackage Pda_Membership_Integration/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Pda_Membership_Integration
 * @subpackage Pda_Membership_Integration/includes
 * @author     BWPS <hello@preventdirectaccess.com>
 */
class Pda_Membership_Integration {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Pda_Membership_Integration_Loader $loader Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string $plugin_name The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string $version The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'PDA_MEMBERSHIP_NAME_VERSION' ) ) {
			$this->version = PDA_MEMBERSHIP_NAME_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'PDA Membership Integration';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Pda_Membership_Integration_Loader. Orchestrates the hooks of the plugin.
	 * - Pda_Membership_Integration_i18n. Defines internationalization functionality.
	 * - Pda_Membership_Integration_Admin. Defines all hooks for the admin area.
	 * - Pda_Membership_Integration_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pda-membership-integration-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pda-membership-integration-i18n.php';

		/**
		 * Membership constants
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pda-membership-integration-constants.php';

		/**
		 * Membership functions
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pda-membership-integration-functions.php';

		/**
		 * Upgrader class
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pda-membership-integration-upgrader.php';

		/**
		 * Required Ymese WP Plugin SDK
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'yme-wp-plugins-sdk/require.php';

		/**
		 * Plugin update checker
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'yme-plugin-update-checker/plugin-update-checker.php';

		/**
		 * Plugin utils
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pda-membership-integration-utils.php';

		/**
		 * Render notification
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . '/includes/views/view-pda-membership-integration-no-membership-plugins.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-pda-membership-integration-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-pda-membership-integration-public.php';

		$this->loader = new Pda_Membership_Integration_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Pda_Membership_Integration_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Pda_Membership_Integration_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {
		$plugin_admin = new Pda_Membership_Integration_Admin( $this->get_plugin_name(), $this->get_version() );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
		$this->loader->add_action( 'admin_notices', $plugin_admin, 'check_plugin_pda_and_pap_activate' );
		$this->loader->add_action( 'wp_ajax_update_membership_integration', $plugin_admin, 'so_wp_ajax_update_membership_integration' );

		// All hook membership integration for pda v3
		$this->loader->add_action( 'pda_v3_membership_integration_settings', $plugin_admin, 'render_memberships_integration_settings' );

		// All hook membership integration for ppp
		$this->loader->add_action( 'ppp_membership_integration_settings', $plugin_admin, 'render_memberships_integration_settings' );
		$this->loader->add_filter( 'ppp_check_user_access_manager', $plugin_admin, 'is_in_user_access_manager' );
		$this->loader->add_filter( 'ppp_check_membership2', $plugin_admin, 'is_in_membership_2' );
		$this->loader->add_filter( 'ppp_check_memberships_pro', $plugin_admin, 'is_in_paid_memberships_pro' );
		$this->loader->add_filter( 'ppp_check_woo_memberships', $plugin_admin, 'is_in_woo_memberships' );
		$this->loader->add_filter( 'ppp_check_woo_subscriptions', $plugin_admin, 'is_in_woo_subscriptions' );

		// Load js and css for setting membership integration in pda gold and pages & posts plugin
		$this->loader->add_action( 'load_js_css_for_pda_integration_memberships', $plugin_admin, 'integration_memberships_load_js_css' );

		$this->loader->add_action( 'rest_api_init', $plugin_admin, 'pda_membership_integration_rest_api_init_cb' );
		$this->loader->add_filter( 'bulk_actions-upload', $plugin_admin, 'membership_integration_custom_bulk_actions' );
		$this->loader->add_filter( 'handle_bulk_actions-upload', $plugin_admin, 'membership_integration_bulk_action_handler', 12, 3 );
		$this->loader->add_action( 'admin_print_footer_scripts', $plugin_admin, 'handle_footer_scripts', 999 );

		// Migrate data
		$this->loader->add_action( 'admin_init', $plugin_admin, 'pda_memberships_migrate_data' );

		$this->loader->add_action( 'pda_get_category_allow_plan_in_ar_member', $plugin_admin, 'get_all_category_allow_plan' );

		// Set FAP for Memberships
		$this->loader->add_action( 'pda_gold_handle_after_set_fap', $plugin_admin, 'pda_memberships_handle_after_pda_gold_set_fap' );

		$this->loader->add_filter( 'puc_request_info_options-pda-membership-integration', $plugin_admin, 'puc_add_token_to_headers', 10 ,1 );


	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Pda_Membership_Integration_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Pda_Membership_Integration_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
