<?php

class PDA_S3_Helper {
	/**
	 * @var Pda_v3_Gold_Helper
	 */
	private $gold_helpers;

	/**
	 * @var PDA_S3_UTILS
	 */
	private $s3_utils;

	/**
	 * @var Aws_Helper
	 */
	private $aws_helper;

	/**
	 * @var PDA_v3_Gold_Repository
	 */
	private $pda_gold_repository;

	/**
	 * @var Pda_Gold_Functions
	 */
	private $gold_functions;


	/**
	 * PDA_S3_Helper constructor.
	 */
	public function __construct() {
		$this->gold_helpers        = new Pda_v3_Gold_Helper();
		$this->s3_utils            = new PDA_S3_UTILS();
		$this->aws_helper          = new \PDAS3\Providers\Aws_Helper();
		$this->pda_gold_repository = new PDA_v3_Gold_Repository();
		$this->gold_functions      = new Pda_Gold_Functions();
	}

	/**
	 * Get object class.
	 *
	 * @return bool|PDA_S3_Helper
	 */
	public static function get() {
		if ( ! class_exists( 'Pda_v3_Gold_Helper' ) || ! class_exists( 'PDA_v3_Gold_Repository' ) || ! class_exists( 'Pda_Gold_Functions' ) ) {
			return false;
		}

		return new PDA_S3_Helper();
	}

	/**
	 * Get attachment ID by File URL.
	 *
	 * @param string  $file_url File URL.
	 * @param boolean $is_image Is image type.
	 *
	 * @return bool|array
	 */
	public function get_attachment_id_by_url( $file_url, $is_image = false ) {
		$wp_upload_dir = wp_upload_dir();
		$baseurl       = $wp_upload_dir['baseurl'];

		// Split url path and query parameters.
		$url_pattern = explode( '?', $file_url );

		// Get URL Path with not query parameters.
		$url = $url_pattern[0];

		$current_file_path = str_replace( $baseurl . '/', '', $url );

		if ( ! method_exists( $this->gold_helpers, 'attachment_image_url_to_post' ) ) {
			return false;
		}

		$pda_baseurl = rtrim( $baseurl, '/' ) . '/_pda';

		// Get attachment object by original url.
		if ( $is_image ) {
			// Handle only image types.
			$attachment_id = $this->get_attachment_id_by_image_url( $file_url, $baseurl, $current_file_path, $pda_baseurl );
		} else {
			// Handle all file types.
			$attachment_id = $this->get_attachment_id_by_default_url( $file_url, $baseurl, $current_file_path, $pda_baseurl );
		}

		if ( empty( $attachment_id ) ) {
			return false;
		}

		list( $size, $url_file ) = $this->gold_helpers->get_image_size_of_link( $url );

		return array(
			'attachment_id' => $attachment_id,
			'size'          => $size,
		);
	}

	/**
	 * Check permission of a attachment.
	 *
	 * @param integer $post_id Post ID.
	 *
	 * @return bool
	 */
	public function is_accessible_url( $post_id ) {
		if ( ! method_exists( $this->gold_functions, 'check_file_access_permission_for_post' ) ) {
			return false;
		}

		// Check file is synced and protected by PDA.
		if ( ! pda_s3_check_file_synced_s3( $post_id ) || ! $this->is_protected_file( $post_id ) ) {
			return false;
		}

		// Check permission.
		return $this->gold_functions->check_file_access_permission_for_post( $post_id );
	}

	/**
	 * Check file is protected.
	 *
	 * @param integer $post_id Post ID.
	 *
	 * @return bool
	 */
	public function is_protected_file( $post_id ) {
		if ( ! method_exists( $this->pda_gold_repository, 'is_protected_file' ) ) {
			return false;
		}

		return $this->pda_gold_repository->is_protected_file( $post_id );
	}

	/**
	 * Convert file URL to signed URL.
	 *
	 * @param string  $file_url     File URL.
	 * @param integer $expired_time Time die.
	 *
	 * @return bool|string
	 */
	public function convert_url_to_signed_url( $file_url, $expired_time ) {
		$extension = wp_check_filetype( $file_url );
		$is_image  = false !== $extension['type'] && false !== strpos( $extension['type'], 'image' );
		$result    = $this->get_attachment_id_by_url( $file_url, $is_image );
		if ( false === $result ) {
			return false;
		}

		// Check permission for attachment.
		if ( ! $this->is_accessible_url( $result['attachment_id'] ) ) {
			return false;
		}

		// Handle with file size.
		if ( $is_image && ! empty( $result['size'] ) ) {

			// Convert -WWWxHHH to [WWW, HHH].
			$size = explode( 'x', ltrim( $result['size'], '-' ) );

			// Get size name ( thumbnail, original ).
			$size = pda_s3_convert_dimensions_to_size_name( $result['attachment_id'], $size );

			return $this->s3_utils->create_presigned_url_for_image_sizes( $file_url, $result['attachment_id'], $size, $expired_time );
		}

		return $this->s3_utils->create_presigned_url( $result['attachment_id'], $expired_time );
	}

	/**
	 * Convert aws URL to signed URL.
	 *
	 * @param string  $aws_url      AWS URL.
	 * @param integer $expired_time Expired time.
	 *
	 * @return bool|false|string
	 */
	public function convert_aws_url_to_signed_url( $aws_url, $expired_time ) {
		return $this->aws_helper->create_read_presigned_url( $aws_url, $expired_time );
	}

	/**
	 * Convert all valid URLs to signed URLs
	 *
	 * @param string $value Value of field.
	 *
	 * @return string
	 */
	public function hand_urls_for_content( $value ) {
		if ( ! method_exists( $this->gold_helpers, 'get_urls_of_image' ) || ! method_exists( $this->gold_helpers, 'get_urls_of_a_tag' )
		) {
			return false;
		}

		// Extract URLs from content of field.
		$image_urls = $this->gold_helpers->get_urls_of_image( $value );
		$a_urls     = $this->gold_helpers->get_urls_of_a_tag( $value );
		$urls       = array_unique( array_merge( $image_urls, $a_urls ) );
		if ( empty( $urls ) ) {
			return $value;
		}

		$domain = wp_parse_url( home_url(), PHP_URL_HOST );
		// Get expired time of signed URL.
		$expired_time = pda_s3_get_setting( PdaS3Constants::PDAS3_EXPIRED_TIME_FOR_SIGNED_URL );
		if ( false === $expired_time ) {
			$expired_time = PdaS3Constants::PDAS3_DEFAULT_EXPIRED_TIME;
		}

		$urls_object = array_map(
			function ( $url ) use ( $domain, $expired_time ) {
				$image_domain = wp_parse_url( $url, PHP_URL_HOST );
				$result       = array(
					'new_url'        => $url,
					'url_to_replace' => $url,
				);

				/**
				 * Handle with file exist on Local.
				 * Else: Handle with S3 URL.
				 * Allow convert to signed URL if file exists on S3 Buckets.
				 */
				if ( $domain === $image_domain ) {
					$result['new_url'] = $this->convert_url_to_signed_url( $url, $expired_time );
				} else {
					$result['new_url'] = $this->convert_aws_url_to_signed_url( $url, $expired_time );
				}

				return $result;
			},
			$urls
		);

		// Replace content by new URL.
		foreach ( $urls_object as $url_object ) {
			if ( ! empty( $url_object['new_url'] ) ) {
				$value = str_replace( $url_object['url_to_replace'], $url_object['new_url'], $value );
			}
		}

		return $value;
	}

	/**
	 * Get attachment ID by image URL.
	 *
	 * @param string $file_url          File URL.
	 * @param string $baseurl           Base URL.
	 * @param string $current_file_path Current file path.
	 * @param string $pda_baseurl       PDA Base URL.
	 *
	 * @return array|false
	 */
	private function get_attachment_id_by_image_url( $file_url, $baseurl, $current_file_path, $pda_baseurl ) {
		$attachment = $this->gold_helpers->attachment_image_url_to_post( $baseurl . '/', $current_file_path );
		if ( empty( $attachment ) && false === strpos( $file_url, $pda_baseurl ) ) {
			$protected_path = '_pda/' . $current_file_path;
			$attachment     = $this->gold_helpers->attachment_image_url_to_post( $baseurl . '/', $protected_path );
		}

		return ! empty( $attachment ) ? $attachment->post_id : false;
	}

	/**
	 * Get attachment ID by URL.
	 *
	 * @param string $file_url          File URL.
	 * @param string $baseurl           Base URL.
	 * @param string $current_file_path Current file path.
	 * @param string $pda_baseurl       PDA Base URL.
	 *
	 * @return integer
	 */
	private function get_attachment_id_by_default_url( $file_url, $baseurl, $current_file_path, $pda_baseurl ) {
		$attachment = attachment_url_to_postid( $baseurl . '/' . $current_file_path );
		if ( empty( $attachment ) && false === strpos( $file_url, $pda_baseurl ) ) {
			$protected_path = '_pda/' . $current_file_path;
			$attachment     = attachment_url_to_postid( $baseurl . '/' . $protected_path );
		}

		return $attachment;
	}

}
