<?php

use PDAS3\Providers\Aws_Helper;
use PDAS3\AsyncTasks\PDA_S3_Offload_Backup_Files;
use PDAS3\AsyncTasks\PDA_S3_Copy_Files_From_Bucket;

if ( ! class_exists( 'PDA_S3_UTILS' ) ) {

	class PDA_S3_UTILS {

		private $aws_helper;

		private $repository;

		function __construct() {
			$this->aws_helper = new Aws_Helper();
			$this->repository = new PDA_S3_Repository();
		}

		/**
		 * Sync file to S3
		 *
		 * @param int $id Attachment's ID.
		 * @param array $metadata Metadata
		 *
		 * @return bool|mixed
		 * @throws \PDAS3\Providers\Exception S3 exception.
		 */
		public function sync_file_to_s3( $id, $metadata = false ) {
			do_action( PdaS3Constants::HOOK_PRE_SYNC, $id );
			$file_path     = get_attached_file( $id );
			$type          = get_post_mime_type( $id );
			$allowed_types = $this->get_allowed_mime_types();
			if ( ! in_array( $type, $allowed_types, true ) ) {
				return false;
			}
			if ( is_file( $file_path ) ) {
				$is_public_default = $this->is_default_public( $id );
				$files_to_remove   = array();
				$root_folder       = pda_s3_get_root_folder( $id );
				try {
					$s3_link = $this->aws_helper->put_file( $file_path, $is_public_default, $root_folder );
					update_post_meta( $id, PdaS3Constants::S3_LINK_META_NAME, $s3_link );
					do_action( PdaS3Constants::HOOK_POST_SYNC, $id, $s3_link );
					$files_to_remove[] = $file_path;
					$this->remove_local_file( $id, $file_path );
				} catch ( \Exception $exception ) {
					return new WP_Error( PdaS3Constants::S3_EXCEPTION, __( 'Failed to sync the file. Please try again or check the configuration.', 'pda-s3' ), array( 'status' => 400 ) );
				}
				$file_paths = $this->get_file_paths( $id, false, $metadata );
				foreach ( $file_paths as $size => $path ) {
					if ( ! in_array( $path, $files_to_remove ) ) {
						try {
							if ( is_file( $path ) ) {
								$this->aws_helper->put_file( $path, $is_public_default, $root_folder );
								$files_to_remove[] = $path;
								$this->remove_local_file( $id, $path );
							}
						} catch ( \Exception $exception ) {
							return new WP_Error( PdaS3Constants::S3_EXCEPTION, __( 'Failed to sync the file. Please try again or check the configuration.', 'pda-s3' ), array( 'status' => 400 ) );
						}
					}

				}

				return $s3_link;
			}

			return false;
		}


		/**
		 * Check option is default public
		 *
		 * @param int $id Post ID.
		 *
		 * @return bool
		 */
		private function is_default_public( $id ) {
			if ( class_exists( 'PDA_v3_Gold_Repository' ) ) {
				$repo = new PDA_v3_Gold_Repository();

				return ! $repo->is_protected_file( $id );
			}
			$settings          = get_option( 'pdas3_options' );
			$is_public_default = true;
			if ( $settings ) {
				$options = unserialize( $settings );
				if ( is_array( $options ) && array_key_exists( 'default_public', $options ) ) {
					$default_public    = $options['default_public'];
					$is_public_default = 'true' === $default_public ? false : true;
				}
			}

			return $is_public_default;
		}

		/**
		 * Check file size for reading to upload
		 *
		 * @param $filename
		 *
		 * @return bool
		 */
		public function check_file_size( $filename ) {
			$file_size  = '314572800';
			$actualSize = filesize( $filename );

			return $actualSize <= $file_size;
		}

		/**
		 * Create a presigned URL
		 *
		 * @param $post_id
		 * @param string $time_die
		 *
		 * @return string
		 */
		public function create_presigned_url( $post_id, $time_die ) {
			$synced_link = urldecode( get_post_meta( $post_id, PdaS3Constants::S3_LINK_META_NAME, true ) );
			$link        = '';
			if ( $synced_link ) {
				$file_path     = get_attached_file( $post_id );
				$original_file = str_replace( '/_pda', '', $file_path );
				$link          = $this->aws_helper->createPresignedUrl( $original_file, $time_die, $post_id );
			}

			return $link;
		}

		/**
		 * Create a pregisned for image file size
		 *
		 * @param $url
		 * @param $attachment_id
		 * @param $size
		 * @param string $time_die
		 *
		 * @return string
		 */
		public function create_presigned_url_for_image_sizes( $url, $attachment_id, $size, $time_die = PdaS3Constants::PDAS3_DEFAULT_EXPIRED_TIME ) {
			$file_paths = $this->get_file_paths( $attachment_id, false );
			if ( isset( $file_paths[ $size ] ) ) {
				$file_path = $file_paths[ $size ];
				$file_path = str_replace( '/_pda', '', $file_path );
				$url       = $this->aws_helper->createPresignedUrl( $file_path, $time_die, $attachment_id );
			}

			return $url;
		}

		public function create_s3_url_for_image_size( $url, $attachment_id ) {
			$original_url = wp_get_attachment_url( $attachment_id );
			$filename     = basename( $original_url );
			$subsize_name = basename( $url );

			return str_replace( $filename, $subsize_name, $original_url );
		}

		/**
		 * Delete file from S3
		 *
		 * @param int $post_id Attachment's ID.
		 *
		 * @return bool|mixed
		 * @throws Exception Exception.
		 */
		public function delete_s3_file( $post_id ) {
			$file_paths = $this->get_file_paths( $post_id, false );
			foreach ( $file_paths as $size => $file_path ) {
				try {
					if ( ! is_file( $file_path ) ) {
						$this->copy_file_from_bucket( $file_path, $post_id );
					}
					if ( is_file( $file_path ) ) {
						if ( pda_s3_is_protected_file( $post_id ) ) {
							$file_path = str_replace( '/_pda', '', $file_path );
						}

						if ( 'original' === $size ) {
							$synced_link   = get_post_meta( $post_id, PdaS3Constants::S3_LINK_META_NAME, true );
							$new_file_path = $this->get_new_file_path( $synced_link, $file_path );
						} else {
							$new_file_path = $file_path;
						}
						try {
							$this->aws_helper->delete_s3_file( $new_file_path, $post_id );
						} catch ( Exception $exception ) {
							error_log( 'Exp: ' . $exception->getMessage() );
							throw $exception;
						}
						if ( 'original' === $size ) {
							delete_post_meta( $post_id, PdaS3Constants::S3_LINK_META_NAME );
						}
					}
				} catch ( Exception $exception ) {
					error_log( 'Exp: ' . $exception->getMessage() );
					throw  $exception;
				}
			}
		}

		/**
		 * Copy file from bucket
		 *
		 * @param string $file_path File path
		 * @param int $post_id Post ID
		 *
		 * @return bool|void
		 * @throws Exception
		 */
		public function copy_file_from_bucket( $file_path, $post_id ) {
			$aws_helper = new Aws_Helper();
			try {
				if ( pda_s3_is_protected_file( $post_id ) ) {
					$before_upload_file_path = str_replace( '/_pda', '', $file_path );
				} else {
					$before_upload_file_path = $file_path;
				}
				$place_to_save = dirname( $file_path );
				if ( ! wp_mkdir_p( $place_to_save ) ) {
					return;
				}
				if ( $aws_helper->copy_object_to_client( $before_upload_file_path, $file_path, $post_id ) === false ) {
					return false;
				}

				return true;
			} catch ( Exception $ex ) {
				error_log( 'DEBUG from bucket: ' . wp_json_encode( $ex ) );
				throw $ex;
			}
		}

		private function get_new_file_path( $synced_link, $old_file_path ) {
			$s3_base_name = basename( $synced_link );
			$wp_base_name = basename( $old_file_path );

			return str_replace( $wp_base_name, $s3_base_name, $old_file_path );
		}

		/**
		 * Function to replace synced and protected file by signedURL
		 *
		 * @param string $content The content.
		 *
		 * @return mixed|null|string|string[]
		 */
		public function replace_protected_files_by_s3_links( $content ) {
			$content = $this->replace_protected_files_link_by_s3_links( $content );
			$content = $this->replace_protected_files_media_by_s3_links( $content );
			$content = $this->replace_protected_video_src_by_s3_links( $content );
			if ( is_plugin_active( 'dflip/dflip.php' ) ) {
				// Replace signed s3 link for Dflip plugin.
				$content = $this->replace_signed_url_for_dflip_plugin( $content );
				$content = $this->replace_signed_url_for_thumbnail_dflip_plugin( $content );
			}

			return $content;
		}

		/**
		 * Replace signed URL for dflip.
		 *
		 * @param string $content The post content.
		 *
		 * @return mixed
		 */
		private function replace_signed_url_for_thumbnail_dflip_plugin( $content ) {
			$thumb        = array();
			$search_thumb = '<div\s[^>]*?thumb\s*=\s*[\'\"]([^\'\"]*?)[\'\"][^>]*?>';
			preg_match_all( "/$search_thumb/iU", $content, $thumb, PREG_PATTERN_ORDER );
			$thumb_link = array_unique( $thumb[1] );

			return $this->handle_content_for_replace_s3_links( $content, $thumb_link, 'thumb' );
		}

		/**
		 * Replace signed URL for dflip.
		 *
		 * @param string $content The post content.
		 *
		 * @return mixed
		 */
		private function replace_signed_url_for_dflip_plugin( $content ) {
			$elements = array();
			$search   = '\"source\":\"([^\"]*)\"';
			preg_match_all( "/$search/iU", $content, $elements, PREG_PATTERN_ORDER );
			$url_file = array_unique( $elements[1] );

			return $this->handle_content_for_replace_s3_links( $content, $url_file, 'source' );

		}

		/**
		 * Replace signed URL for dflip.
		 *
		 * @param string $content The post content.
		 *
		 * @return mixed
		 */
		private function replace_protected_files_media_by_s3_links( $content ) {
			$img         = array();
			$img_pattern = '<img\s[^>]*?src\s*=\s*[\'\"]([^\'\"]*?)[\'\"][^>]*?>';
			preg_match_all( "/$img_pattern/iU", $content, $img );
			$links_file = array_unique( $img[1] );

			return $this->handle_content_for_replace_s3_links( $content, $links_file, 'src' );
		}

		/**
		 * Replace the protected video by s3 link
		 *
		 * @param string $content The content.
		 *
		 * @return mixed
		 */
		private function replace_protected_video_src_by_s3_links( $content ) {
			// classic editor media source.
			$src               = array();
			$video_src_pattern = '<source\s[^>]*?src\s*=\s*[\'\"]([^\'\"]*?)[\'\"][^>]*?>';
			preg_match_all( "/$video_src_pattern/iU", $content, $src );
			$video_src = array_unique( $src[1] );

			// gutenberg editor.
			$video         = array();
			$video_pattern = '<(video|audio)\s[^>]*?src\s*=\s*[\'\"]([^\'\"]*?)[\'\"][^>]*?>';
			preg_match_all( "/$video_pattern/iU", $content, $video );
			$video_links = array_unique( $video[2] );

			$links = array_unique( array_merge( $video_src, $video_links ) );
			// Remove query string part.
			return $this->handle_content_for_replace_s3_links( $content, $links, 'video' );
		}

		/**
		 * Replace signed URL for html link.
		 *
		 * @param string $content The post content.
		 *
		 * @return mixed
		 */
		private function replace_protected_files_link_by_s3_links( $content ) {
			$links = array();
			// https://stackoverflow.com/questions/6711971/regular-expressions-match-anything#6712015.
			$href_pattern = '<a(.*)href=\"([^\"]*)\"((?s).*)>((?s).*)<\/a>';
			preg_match_all( "/$href_pattern/iU", $content, $links );
			$links_file = array_unique( $links[2] );

			return $this->handle_content_for_replace_s3_links( $content, $links_file, 'href' );
		}

		/**
		 * Handle the content for replacing S3 link
		 *
		 * @param string $content The content.
		 * @param array $links_file The link file.
		 * @param string $type The type.
		 *
		 * @return mixed
		 */
		public function handle_content_for_replace_s3_links( $content, $links_file, $type ) {
			foreach ( $links_file as $file ) {
				$massaged_file = pda_s3_remove_size_from_file_name( $file );
				if ( 'source' === $type ) {
					$massaged_file = str_replace( '\\', '', $massaged_file );
				}

				if ( 'video' === $type ) {
					$type          = 'src';
					$massaged_file = strtok( $massaged_file, '?' );
				}

				$post_id = pda_s3_get_post_id_by_cdn_link( $massaged_file );
				if ( false === $post_id ) {
					$post_id = pda_s3_get_post_id_by_s3_link( $massaged_file );
				}

				if ( false === $post_id ) {
					$post_id = pda_s3_get_post_id_by_original_link( $massaged_file );
				}

				if ( false === $post_id ) {
					continue;
				}

				if ( ! class_exists( 'Pda_Gold_Functions' ) && ! class_exists( 'PDA_v3_Gold_Repository' ) ) {
					continue;
				}

				//Condition
				//1. Valid role
				//2. If protected + synced => signed URL
				//3. Otherwise getting in attachment URL
				$repo         = new PDA_v3_Gold_Repository();
				$is_protected = $repo->is_protected_file( $post_id );
				if ( pda_s3_check_file_synced_s3( $post_id ) && $is_protected ) {
					$gold_function = new Pda_Gold_Functions;
					if ( ! method_exists( $gold_function, 'check_file_access_permission_for_post' ) ) {
						return $content;
					}
					$is_valid_role = $gold_function->check_file_access_permission_for_post( $post_id );
					if ( ! $is_valid_role ) {
						return $content;
					}

					$time_die = pda_s3_get_setting( PdaS3Constants::PDAS3_EXPIRED_TIME_FOR_SIGNED_URL );
					if ( false === $time_die ) {
						$time_die = PdaS3Constants::PDAS3_DEFAULT_EXPIRED_TIME;
					}

					$s3_link = $this->create_presigned_url( $post_id, $time_die );
					if ( ! empty( $s3_link ) ) {

						if ( 'source' === $type ) {
							$content = str_replace( '"' . $type . '":"' . $file . '"', '"' . $type . '":"' . $s3_link . '"', $content );
						} else {
							$content = str_replace( $type . '="' . $file . '"', $type . '="' . $s3_link . '"', $content );
						}
					}
				} else {
					$original_link = wp_get_attachment_url( $post_id );
					$content       = str_replace( $file, $original_link, $content );
				}
			}

			return $content;
		}

		public static function IsValidPurchased() {
			$configs   = require( 'class-pda-s3-configs.php' );
			$yme_addon = new YME_Addon( 'pda-s3' );
			$data      = $yme_addon->isValidPurchased( $configs->addonProductId, Yme_Plugin::getLicenseKey( 'pda' ) );

			return ( $data['isValid'] === true );
		}

		public static function addDefaultOptions() {
			if ( is_null( get_option( pda_s3_BWPSConstant::PDAS3_SETTINGS_NAME, null ) ) ) {
				$default_options = array(
					"is_aws_default"           => "true",
					"mg_time_die_private_link" => "1 minutes"
				);
				update_option( pda_s3_BWPSConstant::PDAS3_SETTINGS_NAME, serialize( $default_options ), 'no' );
			}
		}

		/**
		 * Get allowed mime types.
		 *
		 * @return mixed
		 */
		public function get_allowed_mime_types() {
			return apply_filters( 'pda_s3_allowed_mime_types', get_allowed_mime_types() );
		}

		public function get_file_prefix( $post_id ) {
			return $this->get_default_media_file_prefix( $post_id );
		}

		private function get_default_media_file_prefix( $post_id ) {
			$prefix = '';
			$subdir = $this->get_year_month_directory_name( $post_id );
			$prefix .= $subdir;

			return pda_s3_trailingslash( $prefix );
		}

		private function get_year_month_directory_name( $post_id ) {
			$time = get_post_field( 'post_date', $post_id );

			if ( ! $time ) {
				$time = current_time( 'mysql' );
			}

			$y      = substr( $time, 0, 4 );
			$m      = substr( $time, 5, 2 );
			$subdir = "/$y/$m";

			if ( false === strpos( $subdir, '//' ) ) {
				return $subdir;
			}

			return '';
		}

		/**
		 * @param $id
		 * @param $filePath
		 * @param bool $check_setting
		 *
		 * @return bool
		 */
		public function remove_local_file( $id, $filePath, $check_setting = true ) {
			$remove_local_files = true;
			if ( $check_setting ) {
				$remove_local_files = pda_s3_get_setting( 'remove_local_file' ) === 'true';
			}

			if ( $remove_local_files ) {
				if ( ! unlink( $filePath ) ) {
					$message = 'Error when removing the local file';

					if ( ! file_exists( $filePath ) ) {
						$message = "Error removing local file. Couldn't find the file at ";
					} else if ( ! is_writable( $filePath ) ) {
						$message = 'Error removing local file. Ownership or permissions are mis-configured for ';
					}

					error_log( 'Remove local file: ' . $message );

					return false;
				}
			}

			return true;
		}

		public function check_file_is_deleted( $post_id ) {
			$file_path = get_attached_file( $post_id );
			if ( ! is_file( $file_path ) ) {
				return true;
			}

			return false;
		}

		/**
		 *
		 * @return PDA_Logger
		 */
		public static function Logger() {
			if ( class_exists( 'PDA_Logger' ) ) {
				return new PDA_Logger();
			} else {
				return new PDA_S3_Logger();
			}
		}

		/**
		 * Get file paths for the attachment.
		 *
		 * @param int $attachment_id The attachment's ID
		 * @param bool $exists_locally
		 * @param array|bool $meta Meta data
		 *
		 * @return array
		 */
		public function get_file_paths( $attachment_id, $exists_locally = true, $meta = false ) {
			$file_path = get_attached_file( $attachment_id, true );
			$paths     = array(
				'original' => $file_path,
			);

			if ( false === $meta ) {
				$meta = get_post_meta( $attachment_id, '_wp_attachment_metadata', true );
			}

			if ( empty( $meta ) ) {
				return $paths;
			}

			if ( is_wp_error( $meta ) ) {
				return $paths;
			}

			$file_name = wp_basename( $file_path );

			// Handle for edited file
			if ( isset( $meta['file'] ) ) {
				$paths['file'] = str_replace( $file_name, wp_basename( $meta['file'] ), $file_path );
			}

			// Thumb
			if ( isset( $meta['thumb'] ) ) {
				$paths['thumb'] = str_replace( $file_name, $meta['thumb'], $file_path );
			}

			//Sizes
			if ( isset( $meta['sizes'] ) ) {
				foreach ( $meta['sizes'] as $size => $file ) {
					if ( isset( $file['file'] ) ) {
						$paths[ $size ] = str_replace( $file_name, $file['file'], $file_path );
					}
				}
			}

			//Hook allow 3rd party to add files to be uploaded.
			$paths = apply_filters( PdaS3Constants::HOOK_ATTACHMENT_FILE_PATHS, $paths, $attachment_id, $meta );

			$paths = array_unique( $paths );

			if ( $exists_locally ) {
				$paths = array_filter( $paths, function ( $path ) {
					return file_exists( $path );
				} );
			}

			return $paths;
		}

		/**
		 * Handle unique name in bucket
		 *
		 * @param string $filename File name
		 * @param string $ext File extension
		 * @param string $dir File directory
		 * @param bool $include_domain
		 *
		 * @return mixed
		 */
		public function handle_unique_name_in_bucket( $filename, $ext, $dir, $include_domain = true ) {
			$filepath   = trailingslashit( $dir ) . $filename;
			$aws_helper = new Aws_Helper();
			$domain     = $include_domain ? '/' . $_SERVER['HTTP_HOST'] : '';
			$key        = trailingslashit( PdaS3Constants::PDA_S3_PROTECTED_PUBLIC_FOLDER . $domain ) . $aws_helper->getFileKey( $filepath );
			if ( ! $aws_helper->is_file_existed( $key ) ) {
				$key = trailingslashit( PdaS3Constants::PDA_S3_PROTECTED_ROOT_FOLDER . $domain ) . $aws_helper->getFileKey( $filepath );
				if ( ! $aws_helper->is_file_existed( $key ) ) {
					return $filename;
				}
			}

			return $aws_helper->resolveDuplicateFile( $filename );
		}

		/**
		 * Sync to s3 by file path
		 *
		 * @param string $filepath File path
		 *
		 * @return bool|mixed
		 * @throws Exception
		 */
		public function sync_to_s3_by_file_path( $filepath ) {
			if ( ! is_file( $filepath ) ) {
				return false;
			}

			$massaged_file = pda_s3_remove_size_from_file_name( $filepath );
			$upload_dir    = wp_upload_dir();
			$basedir       = $upload_dir['basedir'];

			$value   = str_replace( trailingslashit( $basedir ), '', $massaged_file );
			$repo    = new PDA_S3_Repository();
			$post_id = $repo->get_post_id_by_meta_value( '_wp_attached_file', $value );
			if ( false === $post_id ) {
				return false;
			}

			$root_folder = pda_s3_get_root_folder( $post_id );
			$s3_helper   = new Aws_Helper();

			$is_public = pda_s3_is_protected_file( $post_id );

			try {
				return $s3_helper->put_file( $filepath, $is_public, $root_folder );
			} catch ( Exception $ex ) {
				throw $ex;
			}
		}

		/**
		 * Clear CloudFront cache
		 *
		 * @param $path
		 *
		 * @return bool|\PDAS3\Providers\WP_Error
		 */
		public function clear_cloudfront_cache( $path ) {
			$util = new Aws_Helper();
			$id   = $util->get_distribution_id( pda_s3_get_setting( 'cdn' ) );
			if ( $id === false ) {
				return false;
			}

			return $util->create_invalidation( $id, $path );
		}

		public static function offload_backup_files() {
			$files_offloaded = get_option( PdaS3Constants::PDA_S3_OFFLOADED_OPTION );
			if ( ! is_array( $files_offloaded ) ) {
				return false;
			}
			$task_backup = new PDA_S3_Offload_Backup_Files();
			$task_backup->push_to_queue( array( 'files' => $files_offloaded ) );
			$task_backup->save()->dispatch();
		}

		public static function save_offload_backup_files() {
			$task = new PDA_S3_Copy_Files_From_Bucket();
			$task->push_to_queue( "files" );
			$task->save()->dispatch();
		}

		/**
		 * Generate signed URL
		 *
		 * @param string $file_path File path.
		 *
		 * @return bool
		 */
		public function get_signed_url_when_downloading( $file_path ) {
			if ( class_exists( 'PDA_Services' ) && class_exists( 'PDA_v3_Gold_Repository' ) ) {
				$attachment_id = pda_s3_get_post_id_by_cdn_link( $file_path );
				if ( false === $attachment_id ) {
					$attachment_id = pda_s3_get_post_id_by_s3_link( $file_path );
				}
				if ( false === $attachment_id ) {
					$attachment_id = pda_s3_get_post_id_by_original_link( $file_path );
				}
				if ( false === $attachment_id ) {
					return $file_path;
				}
				$repo              = new PDA_v3_Gold_Repository();
				$is_protected_file = $repo->is_protected_file( $attachment_id );
				if ( ! $is_protected_file ) {
					return wp_get_attachment_url( $attachment_id );
				}
				$is_synced_to_s3 = pda_s3_check_file_synced_s3( $attachment_id );
				if ( ! $is_synced_to_s3 ) {
					return wp_get_attachment_url( $attachment_id );
				}
				$time_die     = pda_get_signed_url_timeout();

				return $this->create_presigned_url( $attachment_id, $time_die );
			}

			return false;
		}

		/**
		 * Check whether the SSU plugin version is lower than expectation.
		 *
		 * @param string $version Version to compare
		 *
		 * @return bool
		 */
		public static function is_ssu_lower_or_equal_than_version( $version ) {
			return defined( 'S3_SMART_UPLOAD_VERSION' ) && version_compare(S3_SMART_UPLOAD_VERSION, $version, '<=');
		}
	}

}
