<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://preventdirectaccess.com/extensions/?utm_source=user-website&utm_medium=pluginsite_link&utm_campaign=pda_gold
 * @since      1.0.0
 *
 * @package    pda_s3
 * @subpackage pda_s3/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    pda_s3
 * @subpackage pda_s3/includes
 * @author     BWPS <hello@preventdirectaccess.com>
 */

require_once dirname( __FILE__ ) . '/constants.php';

class pda_s3 {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      pda_s3_Loader $loader Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string $plugin_name The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string $version The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'pda_s3_VERSION' ) ) {
			$this->version = pda_s3_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'PDA S3 Integration';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - pda_s3_Loader. Orchestrates the hooks of the plugin.
	 * - pda_s3_i18n. Defines internationalization functionality.
	 * - pda_s3_Admin. Defines all hooks for the admin area.
	 * - pda_s3_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pda-s3-loader.php';

		/**
		 * Require the background process
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'lib/wp-background-processing/wp-background-processing.php';

		/**
		 * Ymese SDK
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'yme-wp-plugins-sdk/require.php';

		/**
		 * Plugin update checker
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'yme-plugin-update-checker/plugin-update-checker.php';

		/**
		 * The class responsible for defining plugin's constants
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pda-s3-constants.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pda-s3-i18n.php';

		/**
		 * The class responsible for defining the helpers function
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pda-s3-functions.php';

		/**
		 * Async tasks to upload file to s3
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/async-tasks/class-pda-s3-sync-to-s3.php';

		/**
		 * Async tasks to clear CloudFront cache
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/async-tasks/class-pda-s3-clear-cf-cache.php';

		/**
		 * Async tasks to backup Offload files
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/async-tasks/class-pda-s3-offload-backup-files.php';

		/**
		 * Async tasks to Copy files from bucket
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/async-tasks/class-pda-s3-copy-files-from-bucket.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-pda-s3-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-pda-s3-public.php';

		/**
		 * The class responsible for defining plugin's repository
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pda-s3-repository.php';

		/**
		 * The class responsible for setting utils
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pda-s3-settings-util.php';

		/**
		 * The class responsible for aws utils
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pda-s3-aws-utils.php';

		/**
		 * The own plugin logger
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/logger/class-pda-s3-logger.php';

		/**
		 * The class responsible for utils
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pda-s3-utils.php';


		/**
		 * The class responsible for defining API
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pda-s3-api.php';

		/**
		 * The class responsible for defining Helpers
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pda-s3-helper.php';

		$this->loader = new pda_s3_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the pda_s3_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new pda_s3_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new pda_s3_Admin( $this->get_plugin_name(), $this->get_version() );

		//ajax.
		$this->loader->add_action( 'wp_ajax_pda_s3_integration_update_general_settings', $plugin_admin, 'pda_s3_integration_update_general_settings' );
		$this->loader->add_action( 'wp_ajax_update_option_setting_pdas3', $plugin_admin, 'update_option_setting_pdas3' );
		$this->loader->add_action( 'wp_ajax_pda_s3_handle_sync_file', $plugin_admin, 'pda_s3_handle_sync_file' );

		//assets js, css.
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );

		//api.
		$this->loader->add_action( 'rest_api_init', $plugin_admin, 'wps3_rest_api_init_cb', 12, 2 );

		//Posts, pages content.
		$this->loader->add_filter( 'the_content', $plugin_admin, 'render_s3_link', 999999 );

		//Widget.
		$this->loader->add_action( 'delete_attachment', $plugin_admin, 'handle_delete_attachment', 10 );
		$this->loader->add_filter( 'handle_bulk_actions-upload', $plugin_admin, 'pda_s3_bulk_action_handler', 10, 3 );

		//Settings.
		$this->loader->add_action( 'admin_menu', $plugin_admin, 'add_submenu_pda', 11 );

		//3rd party plugins
		//Woo.
		$this->loader->add_filter( 'woocommerce_product_file_download_path', $plugin_admin, 'handle_woo_file_download', 10, 3 );

		$this->loader->add_filter( 'ewww_image_optimizer_post_optimization', $plugin_admin, 'handle_eww_optimization', 10, 3 );
		//Elementor.
		$this->loader->add_action( 'elementor/frontend/the_content', $plugin_admin, 'handle_elementor_frontend_content', 10 );

		//Rewriting URLs.
		$this->loader->add_filter( 'wp_get_attachment_url', $plugin_admin, 'pda_s3_wp_get_attachment_url', 100, 2 );
		$this->loader->add_filter( 'style_loader_src', $plugin_admin, 'resource_replace_css' );
		$this->loader->add_filter( 'script_loader_src', $plugin_admin, 'resource_replace_js' );
		$this->loader->add_filter( 'get_image_tag', $plugin_admin, 'handle_get_image_tag', 100, 6 );
		$this->loader->add_filter( 'wp_get_attachment_image_src', $plugin_admin, 'handle_get_attachment_image_src', 100, 4 );
		$this->loader->add_filter( 'image_get_intermediate_size', $plugin_admin, 'handle_image_get_intermediate_size', 100, 3 );
		$this->loader->add_filter( 'wp_calculate_image_srcset', $plugin_admin, 'handle_calculate_image_srcset', 10, 5 );
		$this->loader->add_filter( 'wp_calculate_image_srcset_meta', $plugin_admin, 'handle_calculate_image_srcset_meta', 10, 4 );

		//Attachments hook
		$this->loader->add_filter( 'wp_update_attachment_metadata', $plugin_admin, 'handle_wp_update_attachment_metadata', 9999, 2 );
		$this->loader->add_filter( 'load_image_to_edit_path', $plugin_admin, 'handle_load_image_to_edit_path', 10, 3 );

		//PDA Gold Hooks
		$this->loader->add_action( 'pda_after_protected', $plugin_admin, 'handle_after_protect_file' );
		$this->loader->add_action( 'pda_after_un_protected', $plugin_admin, 'handle_after_un_protect_file' );
		$this->loader->add_action( 'pda_after_protect_file_when_upload', $plugin_admin, 'handle_pda_after_protect_file_when_upload' );
		$this->loader->add_action( 'pda_show_status_file_in_pda_column', $plugin_admin, 'handle_show_file_status_in_pda_column', 10, 1 );
		$this->loader->add_filter( 'pda_check_s3_has_bucket', $plugin_admin, 'handle_pda_check_s3_has_bucket' );
		$this->loader->add_filter( 'wp_unique_filename', $plugin_admin, 'handle_wp_unique_filename', 10, 4 );

		// Notification
		$this->loader->add_action( 'admin_notices', $plugin_admin, 'pda_s3_admin_notices' );

		// PDA S3 Hooks
		$this->loader->add_filter( PdaS3Constants::HOOK_GET_ATTACHMENT_URL, $plugin_admin, 'pda_s3_apply_cdn', 10, 2 );
		$this->loader->add_filter( 'wp_generate_attachment_metadata', $plugin_admin, 'auto_sync_after_upload', 9999, 2 );
		$this->loader->add_filter( PdaS3Constants::HOOK_AFTER_DELETE_S3_FILE, $plugin_admin, 'handle_after_delete_s3_file', 10, 4 );
		$this->loader->add_filter( 'redirect_s3_signed_url', $plugin_admin, 'handle_redirect_s3_signed_url', 10, 2 );

		// Plugin loaded
		$this->loader->add_action( 'activated_plugin', $plugin_admin, 'handle_plugins_activated', 10, 2 );
		if ( Yme_Plugin_Utils::is_plugin_activated( 'pda_gold' ) === - 1 || Yme_Plugin_Utils::is_plugin_activated( 'pda_v3' ) === - 1 ) {
			$this->loader->add_filter( 'bulk_actions-upload', $plugin_admin, 'pda_s3_add_bulk_action', 10, 2 );
			$this->loader->add_filter( 'media_row_actions', $plugin_admin, 'add_button_handle_sync_or_unsync', 10, 3 );
		}
		$this->loader->add_filter( 'pda_after_move_files', $plugin_admin, 'pda_after_move_files', 10, 2 );

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new pda_s3_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    pda_s3_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
