<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       preventdirectaccess.com
 * @since      1.0.0
 *
 * @package    Pda_Woocommerce
 * @subpackage Pda_Woocommerce/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Pda_Woocommerce
 * @subpackage Pda_Woocommerce/includes
 * @author     BWPS <hello@preventdirectaccess.com>
 */
class Pda_Woocommerce {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Pda_Woocommerce_Loader $loader Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string $plugin_name The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string $version The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'PLUGIN_NAME_VERSION' ) ) {
			$this->version = PLUGIN_NAME_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'PDA Woocommerce Integration';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Pda_Woocommerce_Loader. Orchestrates the hooks of the plugin.
	 * - Pda_Woocommerce_i18n. Defines internationalization functionality.
	 * - Pda_Woocommerce_Admin. Defines all hooks for the admin area.
	 * - Pda_Woocommerce_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pda-woocommerce-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pda-woocommerce-i18n.php';

		/**
		 * The class responsible for ymese plugin's sdk
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'yme-wp-plugins-sdk/require.php';

		/**
		 * The class responsible for utilities functions
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pda-woocommerce-utils.php';

		/**
		 * The class responsible for working around with setting's values
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pda-wc-setting-utils.php';

		/**
		 * Constants
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pda-woocommerce-constants.php';

		/**
		 * Functions
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-pda-woocommerce-functions.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-pda-woocommerce-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-pda-woocommerce-public.php';

		/**
		 * The class responsible for license checker
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'yme-plugin-update-checker/plugin-update-checker.php';

		$this->loader = new Pda_Woocommerce_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Pda_Woocommerce_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Pda_Woocommerce_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Pda_Woocommerce_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
		$this->loader->add_action( 'admin_notices', $plugin_admin, 'check_plugin_ppp_gold_activate' );

		$this->loader->add_action( 'page-post-woocommerce', $plugin_admin, 'add_tab_woo_page_post' );
		$this->loader->add_action( 'wp_ajax_woo_update_settings', $plugin_admin, 'woo_update_settings' );
		$this->loader->add_action( 'woocommerce_before_resend_order_emails', $plugin_admin, 'handle_woocomerce_be4_send_email', 10, 1 );

		$this->loader->add_action( 'woocommerce_payment_complete', $plugin_admin, 'handle_woocommerce_payment_complete', 10, 1 );

		$this->loader->add_action( 'pdav3_woocommerce_integration_settings', $plugin_admin, 'pda_woocommerce_integration_render_settings', 10, 1 );
		$this->loader->add_action( 'load_js_css_for_pda_integration_woocommerce', $plugin_admin, 'load_js_css_for_setting_woo', 10, 1 );
		$this->loader->add_action( 'wp_ajax_woo_update_settings_pdav3', $plugin_admin, 'woo_update_settings_pdav3' );
		$this->loader->add_action( 'woocommerce_thankyou', $plugin_admin, 'block_access_to_woo_order_by_ip', 10, 1 );
		$this->loader->add_action( 'woocommerce_admin_order_data_after_order_details', $plugin_admin, 'add_list_ip_access_after_shipping_item', 10, 1 );
		$this->loader->add_action( 'woocommerce_download_file_force', $plugin_admin, 'handle_woocommerce_download_file', 9, 2 );
		$this->loader->add_action( 'woocommerce_download_file_xsendfile', $plugin_admin, 'handle_woocommerce_download_file', 9, 2 );
		$this->loader->add_action( 'woocommerce_download_file_redirect', $plugin_admin, 'handle_woocommerce_download_file', 9, 2 );

		$this->loader->add_shortcode( 'pda_woo_products', $plugin_admin, 'display_all_product_woo' );

		$order_status = PDA_WC_Setting_Utils::get_settings( 'ppp_templates' );

		if ( is_array( $order_status ) ) {
			foreach ( $order_status as $status ) {
				$tmp    = str_replace( "wc-", "", $status );
				$action = "woocommerce_order_status_$tmp";

				$this->loader->add_action( $action, $plugin_admin, 'handle_order_completed', 10, 2 );
			}
		}

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Pda_Woocommerce_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Pda_Woocommerce_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
