<?php

/**
 * S3 helper functions
 *
 * @link       buildwps.com
 * @since      1.0.0
 *
 * @package    Protect_Ur_Videos
 * @subpackage Protect_Ur_Videos/includes
 */

/**
 * S3 helper functions
 *
 * Defines functions to work with S3.
 *
 * @since      1.0.0
 * @package    Protect_Ur_Videos
 * @subpackage Protect_Ur_Videos/includes
 * @author     Buildwps <gaupoit@outlook.com>
 */
class Aws_S3_Helper {
	/**
	 * S3 object
	 *
	 * @var \BWPS\Video\AWS\S3\S3Client
	 */
	protected $s3;
	/**
	 * S3 configurations
	 *
	 * @var mixed|null
	 */
	protected $configs = null;

	/**
	 * Aws_S3_Helper constructor.
	 */
	public function __construct() {

		/**
		 * Require the Composer autoloader
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'vendor/autoload.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'vendor/Aws3/aws-autoloader.php';

		$configuration = new Wp_Protect_Video_Configuration();
		$this->configs = $configuration->getAwsConfigs();

		// Try to get settings from PDA S3 if user has been configured yet.
		if ( ! $this->configs ) {
			$this->configs = $this->pda_s3_get_aws_settings();
		}

		if ( $this->configs ) {
			$this->s3 = $this->get_s3_client(
				array(
					'region'     => $this->configs->region,
					'aws_key'    => $this->configs->aws_key,
					'aws_secret' => $this->configs->aws_secret,
				)
			);
		}
	}

	/**
	 * Config AWS SDK.
	 *
	 * @param array $configs AWS Config.
	 *
	 * @return BWPS\Video\Aws3\Aws\S3\S3Client
	 */
	private function get_s3_client( $configs ) {
		return new BWPS\Video\Aws3\Aws\S3\S3Client(
			[
				'version'     => 'latest',
				'region'      => $configs['region'],
				'credentials' => [
					'key'    => $configs['aws_key'],
					'secret' => $configs['aws_secret'],
				],
			]
		);
	}

	/**
	 * Check file size for reading to upload
	 *
	 * @param  string $filename File name.
	 *
	 * @return bool
	 */
	private function check_file_size( $filename ) {
		$file_size   = PWV_Constants::DEFAULT_MAX_FILE_SIZE;
		$actual_size = filesize( $filename );

		return $actual_size <= $file_size;
	}

	/**
	 * Upload to s3 private bucket
	 *
	 * @param string $wp_url File path.
	 *
	 * @return mixed|null
	 * @throws Exception Exception that cannot upload the files more than 300mb.
	 */
	public function upload_to_s3( $wp_url ) {
		$file_path = str_replace( site_url(), untrailingslashit( get_home_path() ), $wp_url );
		if ( is_file( $file_path ) ) {
			if ( $this->check_file_size( $file_path ) ) {
				$file_name     = basename( $file_path );
				$customer_guid = get_option( 'puv_customer_guid' );
				$prefix_key    = ! empty( $this->configs->prefix_key ) ? $this->configs->prefix_key . "/" : "";
				$key           = $prefix_key . "${customer_guid}/${file_name}";
				$result        = $this->s3->putObject( array(
					'Bucket'     => $this->configs->bucket,
					'Key'        => $key,
					'SourceFile' => $file_path,
					'region'     => $this->configs->region,
				) );

				return $result->get( 'ObjectURL' );
			} else {
				throw new Exception( "Please upload the file which's size is less than 300mb" );
			}
		}
	}

	public function testConfiguration( $options ) {
		try {
			$s3 = new BWPS\Video\Aws3\Aws\S3\S3Client( [
				'version'     => 'latest',
				'region'      => $options['region'],
				'credentials' => [
					'key'    => $options['aws_key'],
					'secret' => $options['aws_secret'],
				],
			] );

			return $s3->doesBucketExist( $options['bucket'] );
		} catch ( Exception $ex ) {
			return new WP_Error( 'wrong_aws_configration_', __( 'Please double check your aws configuration at the plugin setting', 'protect-wp-video' ), array( 'status' => 400 ) );
		}
	}

	/**
	 * @param $filePath
	 * @param $time_die
	 * @param $custom_opts
	 *
	 * @return string
	 * empty string if user have never did the aws configuration
	 */
	public function create_presigned_url( $filePath, $time_die, $custom_opts = array() ) {

		if ( is_null( $this->s3 ) ) {
			return "";
		}

		$key          = urldecode( $this->get_file_key( $filePath ) );
		$obj_opts     = array_merge( array(
			'Bucket' => $this->configs->bucket,
			'Key'    => $key,
		), $custom_opts );
		$cmd          = $this->s3->getCommand( 'GetObject', $obj_opts );
		$request      = $this->s3->createPresignedRequest( $cmd, $time_die );
		$presignedUrl = (string) $request->getUri();

		return $presignedUrl;

	}

	/**
	 * Create the pre-signed URL.
	 * If plugin is not config then trying to using from PDA S3 plugin.
	 *
	 * @param string $url    File URL.
	 * @param string $expiry Deadline time.
	 *
	 * @return string|false empty string if user have never did the aws configuration
	 */
	public function create_read_presigned_url( $url, $expiry ) {
		if ( is_null( $this->s3 ) || ! $url ) {
			return false;
		}
		$url_parse = wp_parse_url( $url );

		/**
		 * False on parse failure; Array of URL components on success;
		 * When a specific component has been requested:
		 * null if the component doesn't exist in the given URL;
		 * a string or - in the case of PHP_URL_PORT - integer when it does.
		 */
		if ( ! $url_parse ) {
			return false;
		}

		// Do nothing if cannot parse the URL.
		if ( false === $url_parse ) {
			return false;
		}

		// URL should host from amazon.
		if ( false === strpos( $url_parse['host'], 'amazonaws.com' ) ) {
			return false;
		}

		// URL should match with bucket from Video Config.
		if ( false === strpos( $url, $this->configs->bucket ) ) {
			return false;
		}

		$file_name = ltrim( $url_parse['path'], '/' );

		$cmd = $this->s3->getCommand(
			'GetObject',
			[
				'Bucket' => $this->configs->bucket,
				'Key'    => urldecode( $file_name ),
			]
		);

		$request = $this->s3->createPresignedRequest( $cmd, $expiry );

		return strval( $request->getUri() );
	}

	protected function get_file_key( $file_url ) {
		$file_key = parse_url( $file_url )['path'];
		$bucket   = $this->configs->bucket;
		$file_key = str_replace( "/$bucket/", '', $file_key );
		$file_key = ltrim( $file_key, '/' );

		return $file_key;
	}

	/**
	 * Get AWS settings from PDA S3 plugin.
	 */
	private function pda_s3_get_aws_settings() {
		// Check PDA S3 activate PDA S3 and method is available.
		if ( ! method_exists( 'PDA_S3_Settings_Util', 'getAwsConfigs' ) ) {
			return $this->configs;
		}

		// Try to get AWS settings from PDA S3.
		$pda_s3_settings = new \PDA_S3_Settings_Util();
		$configs         = $pda_s3_settings->getAwsConfigs();

		// Have user configured in PDA S3 settings ?.
		if ( empty( $configs ) ) {
			return $this->configs;
		}

		return (object) array(
			'region'     => $configs['region'],
			'aws_key'    => $configs['aws_key'],
			'aws_secret' => $configs['aws_secret'],
			'bucket'     => $configs['bucket'],
			'prefix_key' => '',
			'domain'     => '',
		);
	}

}
