<?php

if ( ! class_exists( "Pda_Ip_Block_Services" ) ) {
	/**
	 * Class Pda_Ip_Block_Services
	 */
	class Pda_Ip_Block_Services {

		/**
		 * Repository class
		 *
		 * @var Pda_Ip_Block_Repository|null Repository.
		 */
		private $repository;

		/**
		 * Pda_Ip_Block_Services constructor.
		 *
		 * @param null $repository Repository class.
		 */
		public function __construct( $repository = null ) {
			if ( is_null( $repository ) ) {
				$repository = new Pda_Ip_Block_Repository();
			}
			$this->repository = $repository;
		}

		/**
		 * Check File access permission for User
		 * If FAP in PDA is true => return true
		 * If not isset value => return false
		 * If user type is no-users or default => return false
		 * If not isset users => return false
		 * If current user have FAP => return true
		 *
		 * @param bool   $fap           file access permission.
		 * @param string $attachment_id attachment id.
		 *
		 * @return bool
		 */
		public function check_fap_for_user( $fap, $attachment_id ) {
			if ( $fap ) {
				return true;
			}
			$data = $this->repository->get_user_access_by_post_id( $attachment_id );
			if ( ! isset( $data['type']['value'] ) ) {
				return false;
			}
			$type = $data['type']['value'];
			if ( 'no-users' === $type || 'default' === $type ) {
				return false;
			}
			if ( ! isset( $data['users'] ) ) {
				return false;
			}
			foreach ( $data['users'] as $user ) {
				if ( ! isset( $user['value'] ) ) {
					continue;
				}
				if ( get_current_user_id() === (int) $user['value'] ) {
					return true;
				}
			}

			return false;
		}

		/**
		 * Search users by search key request on client
		 *
		 * @param string $search_key search key on client.
		 *
		 * @return array
		 */
		public function search_users( $search_key ) {
			$users = $this->repository->get_user_info_by_user_login( $search_key );

			return array_map(
				function ( $user ) {
					return array(
						'value' => $user->ID,
						'label' => $user->user_login,
					);
				},
				$users
			);
		}

		/**
		 * Get user access by post id
		 *
		 * @param string|int $post_id post id.
		 *
		 * @return mixed
		 */
		public function get_user_access( $post_id ) {
			$results = $this->repository->get_user_access_by_post_id( $post_id );
			if ( defined( 'PDA_GOLD_V3_VERSION' ) ) {
				$results['is_support_pda'] = version_compare( PDA_GOLD_V3_VERSION, '3.1.2.3' ) > 0;
			} else {
				$results['is_support_pda'] = false;
			}

			return $results;
		}

		/**
		 * TODO: Need to apply the UT after complete task
		 * Handle File Access Permission follow IP Address
		 *
		 * @param bool       $default_fap File Access Permission default in PPP Gold or other plugin handled before.
		 * @param int|string $post_id     The post id.
		 *
		 * @return bool
		 */
		public function ppp_handle_fap_follow_ip( $default_fap, $post_id ) {
			$ip_referer = $this->get_ip_referer();
			if ( false === $ip_referer ) {
				return $default_fap;
			}
			$fap_type = get_post_meta( $post_id, Pda_Ip_Block_Constants::IP_BLOCK_FAP_TYPE, true );
			if ( empty( $fap_type ) || 'default_setting' === $fap_type['value'] ) {
				if ( $this->handle_case_default_setting( $ip_referer ) ) {
					return true;
				}
			} elseif ( 'custom_ip' === $fap_type['value'] ) {
				if ( $this->handle_case_custom_ip( $post_id, $ip_referer ) ) {
					return true;
				}
			}

			return $default_fap;
		}

		/**
		 * Handle IP Restriction with case Default setting.
		 *
		 * @param string $ip_referer IP referrer.
		 *
		 * @return bool
		 */
		public function handle_case_default_setting( $ip_referer ) {
			$list_ip = ip_block_list_ip_allow_access_protected_post();
			if ( empty( $list_ip ) ) {
				return false;
			}

			return $this->is_valid_referer_ip( $list_ip, $ip_referer );
		}

		/**
		 * Handle IP Restriction with case custom IP.
		 *
		 * @param int|string $post_id    The post ID.
		 * @param string     $ip_referer IP referrer.
		 *
		 * @return bool
		 */
		public function handle_case_custom_ip( $post_id, $ip_referer ) {
			$all_ip  = $this->repository->get_data_by_post_id( $post_id );
			$list_ip = explode( ';', $all_ip->ip_block );

			return $this->is_valid_referer_ip( $list_ip, $ip_referer );
		}

		/**
		 * Check referer IP in list IPs allow access protected page/post
		 *
		 * @param array  $list_ip    List IPs allow access protected page/post.
		 * @param string $ip_referer IP referer.
		 *
		 * @return bool
		 */
		public function is_valid_referer_ip( $list_ip, $ip_referer ) {
			foreach ( $list_ip as $ip ) {
				if ( $ip === $ip_referer ) {
					return true;
				}

				$custom_id = strpos( $ip, '*' );
				if ( false !== $custom_id && $this->ppp_is_valid_ip( $ip, $ip_referer ) ) {
					return true;
				}
			}

			return false;
		}

		/**
		 * Check is valid IP
		 *
		 * @param string $ip_access  IP allow access view article.
		 * @param string $ip_referer IP referrer.
		 *
		 * @return bool
		 */
		public function ppp_is_valid_ip( $ip_access, $ip_referer ) {
			$ips = explode( '.', $ip_access );
			if ( count( $ips ) !== 4 ) {
				return false;
			}
			$every      = '(25[0-5]|2[0-4][0-9]|[01]?[0-9]?[0-9])';
			$ips        = array_map(
				function ( $element ) use ( $every ) {
					if ( '*' === $element ) {
						return $every;
					}

					return $element;
				},
				$ips
			);
			$ip_pattern = '/\b(?:' . $ips[0] . '\.' . $ips[1] . '\.' . $ips[2] . '\.' . $ips[3] . ')\b/';

			return preg_match( $ip_pattern, $ip_referer ) === 1;
		}

		/**
		 * Get IP referer.
		 *
		 * @return mixed
		 */
		public function get_ip_referer() {
			$ip_header_names = array(
				'HTTP_CF_CONNECTING_IP',
				'HTTP_CLIENT_IP',
				'HTTP_X_FORWARDED_FOR',
				'REMOTE_ADDR',
			);

			foreach ( $ip_header_names as $ip_header_name ) {
				if ( ! empty( $_SERVER[ $ip_header_name ] ) ) {
					return $_SERVER[ $ip_header_name ]; // phpcs:ignore
				}
			}

			return false;
		}

		/**
		 * Validate and get parameter in data request from client.
		 *
		 * @param array $data   Data from client.
		 * @param array $params All parameters need to validate.
		 *
		 * @return bool
		 */
		public function is_valid_data( $data, $params ) {
			foreach ( $params as $param ) {
				if ( ! isset( $data[ $param ] ) ) {
					return false;
				}
			}

			return true;
		}

		/**
		 * Update IP FAP for PPP plugin
		 *
		 * @param int|string $post_id  The post ID.
		 * @param array      $fap_type File Access Permission type.
		 * @param array      $ips      IP allow access article.
		 */
		public function ppp_update_ip_fap( $post_id, $fap_type, $ips ) {
			update_post_meta( $post_id, Pda_Ip_Block_Constants::IP_BLOCK_FAP_TYPE, $fap_type );
			$this->repository->update_ip_fap( $post_id, $ips );
		}

		/**
		 * Get IP FAP.
		 *
		 * @param int|string $post_id The pos ID.
		 *
		 * @return array
		 */
		public function ppp_get_ip_fap( $post_id ) {
			$fap_type = get_post_meta( $post_id, Pda_Ip_Block_Constants::IP_BLOCK_FAP_TYPE, true );

			if ( empty( $fap_type ) ) {
				$fap_type = array(
					'value' => 'default_setting',
					'label' => 'Use default setting',
				);
			}
			$data = $this->repository->get_data_by_post_id( $post_id );
			$ips  = isset( $data->ip_block ) && ! empty( $data->ip_block ) ? explode( ';', $data->ip_block ) : array();

			return array(
				'ip_block'    => $ips,
				'ip_fap_type' => $fap_type,
			);
		}

		/**
		 * Response for request.
		 *
		 * @param int    $status  Response status.
		 * @param string $message Message for API.
		 * @param bool   $success Success status.
		 *
		 * @return WP_REST_Response
		 */
		public function send_json( $status = 200, $message = '', $success = true ) {
			return new WP_REST_Response(
				array(
					'isError' => $success,
					'message' => $message,
				),
				$status
			);
		}

		/**
		 * Get folder types by folder selected.
		 *
		 * @param array $selected_folders Folder user select in settings page.
		 *
		 * @return array
		 */
		public function get_folder_types( $selected_folders ) {
			if ( empty( $selected_folders ) ) {
				return array(
					'root'      => array(),
					'wp_upload' => array(),
				);
			}
			$root_folder   = array();
			$upload_folder = array();
			foreach ( $selected_folders as $folder ) {
				if ( 0 === strpos( $folder, '^' ) ) {
					$root_folder[] = str_replace( '^', '', $folder );
				} else {
					$upload_folder[] = $folder;
				}
			}

			return array(
				'root'      => $root_folder,
				'wp_upload' => $upload_folder,
			);
		}

		/**
		 * Render folder protection rules in NGINX server.
		 *
		 * @param array $selected_folders Selected folders from user.
		 *
		 * @return array
		 */
		public function render_folder_protection_nginx_rules( $selected_folders ) {
			$types         = $this->get_folder_types( $selected_folders );
			$root_folder   = $types['root'];
			$upload_folder = $types['wp_upload'];
			$rules         = array();
			if ( ! empty( $root_folder ) ) {
				$root_tmp  = implode( '|', $root_folder );
				$root_tmp  = preg_replace( '!\s+!', '\s+', $root_tmp );
				$root_rule = sprintf( 'rewrite ((%1$s)/.*\.\w+)$ "/index.php?%2$s=$1" last;', $root_tmp, PDA_v3_Constants::$secret_param );
				array_push( $rules, $root_rule );
			}

			if ( ! empty( $upload_folder ) ) {
				$upload_tmp  = implode( '|', $upload_folder );
				$upload_tmp  = preg_replace( '!\s+!', '\s+', $upload_tmp );
				$upload_path = ip_block_get_wp_upload_folder();
				$upload_rule = sprintf( 'rewrite %1$s(/(%2$s)/.*\.\w+)$ "/index.php?%3$s=$1" last;', $upload_path, $upload_tmp, PDA_v3_Constants::$secret_param );
				array_push( $rules, $upload_rule );
			}

			return $rules;
		}

		/**
		 * Check whether the current referral URL is whitelisted.
		 *
		 * @param string $file_folder The file folder.
		 *
		 * @return bool
		 */
		public function is_valid_ref_url_folder_protection( $file_folder ) {
			$referer = wp_get_raw_referer();
			if ( false === $referer ) {
				return false;
			}
			$whitelisted_refs = get_data_protect_folder_type_is_array( Pda_Ip_Block_Constants::PDA_WHITE_LIST_REF_URL_FOLDER_PROTECT );

			if ( ! is_array( $whitelisted_refs ) ) {
				return false;
			}

			$folders = array_column( $whitelisted_refs, 'folder' );
			$idx     = array_search( $file_folder, $folders, true );
			if ( false === $idx ) {
				return false;
			}

			if ( ! isset( $whitelisted_refs[ $idx ]['ref'] ) ) {
				return false;
			}
			$ref_conf = $whitelisted_refs[ $idx ]['ref'];
			$option   = $ref_conf['opt'];

			if ( 'disable' === $option ) {
				return false;
			}

			if ( 'all' === $option ) {
				return true;
			}

			if ( 'custom' === $option && isset( $ref_conf['values'] ) && is_array( $ref_conf['values'] ) ) {
				return $this->is_ref_in_custom_values( $referer, $ref_conf['values'] );
			}

			return false;
		}

		/**
		 * Check in user-defined URLs.
		 *
		 * @param string $ref    Current Ref URL.
		 * @param array  $values User-defined whitelisted URLs.
		 *
		 * @return bool
		 */
		private function is_ref_in_custom_values( $ref, $values ) {
			foreach ( $values as $url ) {
				if ( $ref === $url ) {
					return true;
				}

				// Check whether the last character is *.
				$last_char = substr( $url, - 1 );
				if ( '*' !== $last_char ) {
					continue;
				}

				// Remove the * and compare to ref position in URL.
				$pure_url = substr( $url, 0, - 1 );
				if ( 0 !== strpos( $ref, $pure_url ) ) {
					continue;
				}

				return true;

			}

			return false;
		}

		/**
		 * Checking whether the IP is whitelisted.
		 *
		 * @param array $ips The whitelisted IPs.
		 *
		 * @return bool
		 */
		public function is_whitelisted_ip( $ips ) {
			if ( ! method_exists( 'Pda_v3_Gold_Helper', 'get_instance' ) || ! method_exists( 'Pda_v3_Gold_Helper', 'get_ip' ) ) {
				return false;
			}

			$gold_helpers = Pda_v3_Gold_Helper::get_instance();
			$current_ip   = $gold_helpers->get_ip( $_SERVER );
			foreach ( $ips as $ip ) {
				if ( strpos( $ip, '*' ) !== false && $gold_helpers->valid_ip( $ip, $current_ip ) ) {
					return true;
				}
				if ( $ip === $current_ip ) {
					return true;
				}
			}

			return false;
		}
	}
}
