<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       www.buildwps.com/prevent-direct-access-premium-plugin/?utm_source=user-website&utm_medium=pluginpage&utm_campaign=plugin-author-link
 * @since      1.0.0
 *
 * @package    Wp_Pda_Ip_Block
 * @subpackage Wp_Pda_Ip_Block/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Wp_Pda_Ip_Block
 * @subpackage Wp_Pda_Ip_Block/includes
 * @author     BWPS <hello@ymese.com>
 */
class Wp_Pda_Ip_Block {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Wp_Pda_Ip_Block_Loader $loader Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string $plugin_name The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string $version The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'PDA_IP_BLOCK_VERSION' ) ) {
			$this->version = PDA_IP_BLOCK_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'PDA Access Restriction';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Wp_Pda_Ip_Block_Loader. Orchestrates the hooks of the plugin.
	 * - Wp_Pda_Ip_Block_i18n. Defines internationalization functionality.
	 * - Wp_Pda_Ip_Block_Admin. Defines all hooks for the admin area.
	 * - Wp_Pda_Ip_Block_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wp-pda-ip-block-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wp-pda-ip-block-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-wp-pda-ip-block-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-wp-pda-ip-block-public.php';

		/**
		 * The class responsible for databases class
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wp-pda-ip-block-db.php';

		/**
		 * The class responsible for service class
		 */

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wp-pda-ip-block-repository.php';

		/**
		 * This class responsible for require all sdk helper classes
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'yme-wp-plugins-sdk/require.php';

		/**
		 * Include constants
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wp-pda-ip-block-constants.php';

		/**
		 * The class responsible for helper class
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wp-pda-ip-block-functions.php';

		/**
		 * The class responsible for service class
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wp-pda-ip-block-services.php';

		/**
		 * The class responsible for service class
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/services/class-service-shortcode-integration.php';

		$this->loader = new Wp_Pda_Ip_Block_Loader();
	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Wp_Pda_Ip_Block_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Wp_Pda_Ip_Block_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Wp_Pda_Ip_Block_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );

		$this->loader->add_action( 'admin_notices', $plugin_admin, 'check_plugin_pda_activate' );
		$this->loader->add_action( 'admin_init', $plugin_admin, 'handle_admin_init' );

		$this->loader->add_action( 'pda_ip_block', $plugin_admin, 'render_ip_block' );

		$this->loader->add_filter( 'pda_ip_white_list', $plugin_admin, 'get_ip_white_list' );

		$this->loader->add_filter( 'pdav3_ip_white_list', $plugin_admin, 'get_ip_white_list_for_pdav3' );

		$this->loader->add_action( 'wp_ajax_update_ip_white_list', $plugin_admin, 'so_wp_ajax_update_ip_white_list' );

		$this->loader->add_action( 'rest_api_init', $plugin_admin, 'rest_api_ip_block' );

		$this->loader->add_filter( 'pda_gold_htaccess_contents', $plugin_admin, 'ip_block_htaccess_contents' );

		$this->loader->add_filter( 'pdav3_ip_white_list_website', $plugin_admin, 'get_ip_white_list_website_for_pdav3' );

		// Handle the hook after PDA check file existed.
		$this->loader->add_filter( 'pda_after_check_file_exist', $plugin_admin, 'allow_ip', 20, 2 );

		$this->loader->add_action( 'delete_blog', $plugin_admin, 'delete_table_site_website', 10, 2 );

		// Handle bulk action Assign access permission.
		$this->loader->add_filter( 'bulk_actions-upload', $plugin_admin, 'pda_ip_block_custom_bulk_actions' );
		$this->loader->add_filter( 'handle_bulk_actions-upload', $plugin_admin, 'pda_ip_block_bulk_action_handler', 10, 3 );
		$this->loader->add_action( 'admin_print_footer_scripts', $plugin_admin, 'pda_ip_block_handle_footer_scripts', 999 );

		$this->loader->add_action( 'pda_render_ui_for_protect_folder_tab', $plugin_admin, 'render_ui_for_protect_folder_tab' );
		$this->loader->add_action( 'pda_load_js_css_for_protect_folder_tab', $plugin_admin, 'load_js_css_for_protect_folder_tab' );
		$this->loader->add_action( 'wp_ajax_pda_gold_update_data_for_protect_folder', $plugin_admin, 'pda_gold_update_data_for_protect_folder' );
		$this->loader->add_filter( 'pda_v3_get_rewrite_rules', $plugin_admin, 'pda_add_rules' );
		$this->loader->add_action( 'pda_custom_handle_protected_file', $plugin_admin, 'handle_before_send_file' );
		$this->loader->add_filter( 'puc_request_info_options-pda-ip-block', $plugin_admin, 'puc_add_token_to_headers', 10, 1 );

		// Add action Set FAP.
		$this->loader->add_action( 'pda_gold_handle_after_set_fap', $plugin_admin, 'pda_ip_block_set_fap' );
		$this->loader->add_filter( 'pda_ip_block_handle_set_fap', $plugin_admin, 'pda_handle_set_fap', 10, 2 );
		$this->loader->add_filter( 'pda_file_access_permission', $plugin_admin, 'check_fap_for_ip_block', 10, 2 );

		$this->loader->add_action( 'ppp_gold_add_new_tab', $plugin_admin, 'add_new_tab_for_ppp' );
		$this->loader->add_action( 'ppp_gold_custom_tab', $plugin_admin, 'custom_tab_for_ppp' );
		$this->loader->add_action( 'ppp_gold_render_content_' . Pda_Ip_Block_Constants::PPP_SETTING_TAB, $plugin_admin, 'render_content_access_restriction_tab' );

		// Self hook apply.
		$this->loader->add_filter( Pda_Ip_Block_Constants::HOOK_BEFORE_CHECK_FAP_FOLDER_PROTECTION, $plugin_admin, 'check_ref_url_folder_protection', 10, 2 );
	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Wp_Pda_Ip_Block_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

		$this->loader->add_filter( 'ppp_before_render_content', $plugin_public, 'ppp_handle_fap_follow_ip', 10, 2 );

		$this->loader->add_filter( 'plugins_loaded', $plugin_public, 'handle_plugins_loaded', 5, 2 );
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @return    string    The name of the plugin.
	 * @since     1.0.0
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @return    Wp_Pda_Ip_Block_Loader    Orchestrates the hooks of the plugin.
	 * @since     1.0.0
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @return    string    The version number of the plugin.
	 * @since     1.0.0
	 */
	public function get_version() {
		return $this->version;
	}

}
