<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://preventdirectaccess.com/
 * @since      1.0.0
 *
 * @package    Prevent_Direct_Access_By_S3
 * @subpackage Prevent_Direct_Access_By_S3/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Prevent_Direct_Access_By_S3
 * @subpackage Prevent_Direct_Access_By_S3/includes
 * @author     BWPS <hello@preventdirectaccess.com>
 */
class Prevent_Direct_Access_By_S3 {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Prevent_Direct_Access_By_S3_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'PDA_S3_VERSION' ) ) {
			$this->version = PDA_S3_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'prevent-direct-access-by-s3';

		$this->load_dependencies();
        $this->define_admin_hooks();

        $licenseKey = Yme_Plugin::getLicenseKey('pda-s3');

        if( !empty($licenseKey) ) {
            $this->set_locale();
            $this->define_public_hooks();
        }
	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Prevent_Direct_Access_By_S3_Loader. Orchestrates the hooks of the plugin.
	 * - Prevent_Direct_Access_By_S3_i18n. Defines internationalization functionality.
	 * - Prevent_Direct_Access_By_S3_Admin. Defines all hooks for the admin area.
	 * - Prevent_Direct_Access_By_S3_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

        /**
         * The class responsible for orchestrating the actions and filters of the
         * core plugin.
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-prevent-direct-access-by-s3-loader.php';

		/**
		 * Require the Composer autoloader
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'vendor/autoload.php';

        /**
         * The class responsible for Ymese helper classes
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'yme-wp-plugins-sdk/require.php';


        /**
         * The class responsible for license checker
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'yme-plugin-update-checker/plugin-update-checker.php';

        /**
         * The class to work with DB
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-prevent-direct-access-by-s3-repository.php';

        /**
         * The class responsible for defining all actions that occur in the admin area.
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-prevent-direct-access-by-s3-admin.php';

        /**
         * The class responsible for defining constants in plugin
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-prevent-direct-access-by-s3-constants.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-prevent-direct-access-by-s3-aws-utils.php';

		/**
		 * The class responsible for settings utilities
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-prevent-direct-access-by-s3-settings-util.php';


		$licenseKey = Yme_Plugin::getLicenseKey('pda-s3');

        if( !empty($licenseKey) ) {
            /**
             * The class responsible for defining internationalization functionality
             * of the plugin.
             */
            require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-prevent-direct-access-by-s3-i18n.php';

            /**
             * The class responsible for defining all actions that occur in the public-facing
             * side of the site.
             */
            require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-prevent-direct-access-by-s3-public.php';

            /**
             * The class responsible for rendering wizard pages
             */
            require_once plugin_dir_path( dirname( __FILE__) ) . 'includes/class-prevent-direct-access-s3-setup-wizard.php';

            /**
             * The class responsible for generating htaccess rules
             */
            require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-prevent-direct-access-s3-htaccess.php';

            /**
             * The class responsible for defining APIs
             */
            require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-prevent-direct-access-by-s3-api.php';

            /**
             * The class to init DB
             */
            require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-prevent-direct-access-by-s3-db.php';

            /**
             * The class responsible for rendering views
             */
            require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/views/autoload_view.php';
            /**
             *
             */
            require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-prevent-direct-access-by-s3-utils.php';

        }

        $this->loader = new Prevent_Direct_Access_By_S3_Loader();

    }

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Prevent_Direct_Access_By_S3_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Prevent_Direct_Access_By_S3_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

        $plugin_admin = new Prevent_Direct_Access_By_S3_Admin( $this->get_plugin_name(), $this->get_version() );
        $licenseKey = Yme_Plugin::getLicenseKey('pda-s3');
        $this->loader->add_action('admin_notices', $plugin_admin, 'wps3_admin_notices');
		$this->loader->add_action('admin_menu', $plugin_admin, 'wps3_add_menu_setting');

		$this->loader->add_action('wp_ajax_pda_s3_update_general_settings', $plugin_admin, 'pda_s3_update_general_settings');
		$this->loader->add_action('wp_ajax_pda_s3_update_ip_block', $plugin_admin, 'pda_s3_update_ip_block');
        $this->loader->add_action('wp_ajax_pda_s3_license_checking', $plugin_admin, 'pda_s3_license_checking');
        $this->loader->add_action('wp_ajax_pda_s3_test_configuration', $plugin_admin, 'pda_s3_test_configuration');

        if( !empty($licenseKey) ) {
            $this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
            $this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
            $this->loader->add_action( 'admin_menu', $plugin_admin, 'admin_menu_handler');
            $this->loader->add_action( 'admin_init', $plugin_admin, 'set_custom_cookie' );
            $this->loader->add_action( 'admin_init', $plugin_admin, 'setup_wizard' );
            $this->loader->add_action( 'admin_init', $plugin_admin, 'wps3_htaccess_updated' );
            $this->loader->add_action( 'wp_logout', $plugin_admin, 'unset_custom_cookie' );
            $this->loader->add_filter( 'manage_upload_columns', $plugin_admin, 'wpps3_add_upload_columns' );
            $this->loader->add_action( 'manage_media_custom_column', $plugin_admin, "wpps3_media_custom_column", 10, 2 );
            $this->loader->add_action('rest_api_init', $plugin_admin, 'wps3_rest_api_init_cb', 10, 2);
            $this->loader->add_filter('mod_rewrite_rules', $plugin_admin, 'wps3_customize_htaccess_content');
            $this->loader->add_action( 'delete_attachment', $plugin_admin,'delete_private_link_s3' );
            $this->loader->add_action( 'add_attachment', $plugin_admin,'auto_synced_new_file_to_s3' );

            $this->loader->add_filter( 'cron_schedules', $plugin_admin, 'pda_s3_cron_interval');
            $this->loader->add_action( 'pda_s3_cron_le_hook', $plugin_admin, 'pda_s3_cron_le_exec');
        }

    }

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Prevent_Direct_Access_By_S3_Public( $this->get_plugin_name(), $this->get_version() );

        $this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Prevent_Direct_Access_By_S3_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}